/* 
 * FPLOT - fpout.c
 *
 * Copyright (C) 1997   Michael C. Ring
 *
 * Permission to use, copy, and distribute this software and its
 * documentation for any purpose with or without fee is hereby granted, 
 * provided that the above copyright notice appear in all copies and 
 * that both that copyright notice and this permission notice appear 
 * in supporting documentation.
 *
 * Permission to modify the software is granted, but not the right to
 * distribute the modified code.  Modifications are to be distributed 
 * as patches to released version.
 *  
 * This software is provided "as is" without express or implied warranty.
 */

/*
 *      fpout.c
 *
 *      $Log: fpout.c $
 *      Revision 1.3  1997/11/26 23:21:14  mring
 *      added MAX_LINE_LENGTH
 *
 *      Revision 1.2  1997/11/24 01:34:19  mring
 *      added copyright notice
 *
 *      Revision 1.1  1997/11/23 23:08:25  mring
 *      Initial revision
 *
 */

#include "fplot.h"
#include "glob.h"

/* RCS TRACER */
static  char    
RCS_ID[] = "$Id: fpout.c 1.3 1997/11/26 23:21:14 mring Exp $";

static  FILE    *ptr_out;

/***********************************************************************/
output_epson_lq_plot()
{
if (g_output_file[0] == '\0')
   ptr_out = stdout;
else
  {
   if ((ptr_out = fopen(g_output_file, "wb")) == NULL)
     {
      fprintf(stderr,"Could not open plot file \'%s\'\n",g_output_file);
      exit_program(27);
     }
  }

/* add code here for Epson LQ ... */

if (g_output_file[0] != '\0')
  fclose(ptr_out);
}
/***********************************************************************/
output_epson_fx_plot()
{
if (g_output_file[0] == '\0')
   ptr_out = stdout;
else
  {
   if ((ptr_out = fopen(g_output_file, "wb")) == NULL)
     {
      fprintf(stderr,"Could not open plot file \'%s\'\n",g_output_file);
      exit_program(27);
     }
  }

/* add code here for Epson FX ... */

if (g_output_file[0] != '\0')
  fclose(ptr_out);
}
/***********************************************************************/
output_hpgl2_plot()
{
if (g_output_file[0] == '\0')
   ptr_out = stdout;
else
  {
   if ((ptr_out = fopen(g_output_file, "wb")) == NULL)
     {
      fprintf(stderr,"Could not open plot file \'%s\'\n",g_output_file);
      exit_program(27);
     }
  }

/* add code here for HPGL2 ... */

if (g_output_file[0] != '\0')
  fclose(ptr_out);
}
/***********************************************************************/
output_postscript_plot()
{
if (g_output_file[0] == '\0')
   ptr_out = stdout;
else
  {
   if ((ptr_out = fopen(g_output_file, "w")) == NULL)
     {
      fprintf(stderr,"Could not open plot file \'%s\'\n",g_output_file);
      exit_program(27);
     }
  }

ps_print_header();

ps_print_dict();            /* output the dictionary  */

ps_print_plot();            /* output the actual plot */

ps_print_trailer();

if (g_output_file[0] != '\0')
  fclose(ptr_out);
}
/***********************************************************************/
ps_print_header()
{
fprintf(ptr_out,"%%!PS-Adobe-1.0\n");
fprintf(ptr_out,"%%%%Creator: fplot\n");
fprintf(ptr_out,"%%%%Pages: (atend)\n");
fprintf(ptr_out,"%%%%EndComments\n");
}
/***********************************************************************/
ps_print_dict()
{
fprintf(ptr_out,"/fplot 12 dict def\n");
fprintf(ptr_out,"fplot begin\n");
fprintf(ptr_out,"/ht 320.0 def\n");
fprintf(ptr_out,"/bt 640.0 def\n");          /* 2 * ht */
fprintf(ptr_out,"/N {newpath} bind def\n");
fprintf(ptr_out,"/S {stroke} bind def\n");
fprintf(ptr_out,"/M {moveto} bind def\n");
fprintf(ptr_out,"/L {lineto} bind def\n");
fprintf(ptr_out,"/R {rmoveto} bind def\n");
fprintf(ptr_out,"/T {rlineto} bind def\n");
fprintf(ptr_out,"/D { N M ht 0 R 0 ht sub ht T 0 ht sub 0 ht sub T\n");
fprintf(ptr_out,"ht 0 ht sub T closepath S } def\n");
fprintf(ptr_out,"/B { N M bt 2 div bt 2 div R 0 bt sub 0 T 0 0 bt sub T\n");
fprintf(ptr_out,"bt 0 T closepath fill } def\n");
fprintf(ptr_out,"end\n");
fprintf(ptr_out,"%%%%Page: 1 1\n");
fprintf(ptr_out,"%%%%EndProlog\n");
}
/***********************************************************************/
ps_print_plot()
{
fprintf(ptr_out,"fplot begin\n");
fprintf(ptr_out,"gsave\n");
fprintf(ptr_out,"0.010 0.010 scale\n");
fprintf(ptr_out,"0 setgray\n");

ps_print_box_labels_data();

fprintf(ptr_out,"grestore\n");
fprintf(ptr_out,"end\n");
}
/***********************************************************************/
ps_print_box_labels_data()
{
double  d1, d2, c1, c2, cstart, delta, xdata, ydata, y1slope, y1int,
        y2slope, y2int, xslope, xint;
int     i, j, jdiv, first, dash_on, dash_off, mask, bitmap, ct5, ct150,
        field_count, draw_lines, log_flag;
long    graph_x_low, graph_x_high, graph_y_low, graph_y_high,
        lstr, xcoord, ycoord, border, m, mx, my, local_skip, sample_count;
char    *cp, *cp2;
FILE    *ptr;

/* allocate a string buffer for temp use */
cp   = (char *)memory_allocate(1500 * sizeof(char));
cp2  = cp;
cp2 += 1000;

border = 7200;     /* graph border is 1" scaled 100:1 and 72 points/inch */

/*
 *  calculate graph border and the X, Y1, & Y2 slopes and y-intercepts
 */

graph_x_low  = border;
graph_x_high = (long)(g_papersize_length * 7200 - border + 0.5);

graph_y_high = border;
graph_y_low  = (long)(g_papersize_width * 7200 - border + 0.5);

y1slope = (double)(graph_y_high - graph_y_low) /
                  (g_graph_y1_max - g_graph_y1_min);

y1int   = graph_y_high - y1slope * g_graph_y1_max;

y2slope = (double)(graph_y_high - graph_y_low) /
                  (g_graph_y2_max - g_graph_y2_min);

y2int   = graph_y_high - y2slope * g_graph_y2_max;

xslope  = (double)(graph_x_high - graph_x_low) /
                  (g_graph_x_max - g_graph_x_min);

xint    = graph_x_high - xslope * g_graph_x_max;

/*
 *      draw the box (that the plot will fit inside of)
 */

fprintf(ptr_out,"200 setlinewidth\n");
fprintf(ptr_out,"%ld %ld M\n",graph_y_low,graph_x_low);
fprintf(ptr_out,"%ld %ld L\n",graph_y_high,graph_x_low);
fprintf(ptr_out,"%ld %ld L\n",graph_y_high,graph_x_high);
fprintf(ptr_out,"%ld %ld L closepath S\n",graph_y_low,graph_x_high);

/*
 *      display the labels now
 */

fprintf(ptr_out,"/Helvetica findfont 1400 scalefont setfont\n");

if (g_xlabel[0] != '\0')        /* X Label */
  {
   lstr = convert_string_to_ps(g_xlabel,cp);

   mx  = (graph_x_low + graph_x_high) / 2;
   mx -= lstr / 2;
   my  = graph_y_low + 4600;

   fprintf(ptr_out,"gsave\n");
   fprintf(ptr_out,"%ld %ld M 90 rotate\n",my,mx);
   fprintf(ptr_out,"%s show\n",cp);
   fprintf(ptr_out,"grestore\n");
  }


if (g_y1label[0] != '\0')        /* Y1 Label */
  {
   lstr = convert_string_to_ps(g_y1label,cp);

   mx  = graph_x_low - 4600;
   my  = (graph_y_low + graph_y_high) / 2;
   my += lstr / 2;

   fprintf(ptr_out,"gsave\n");
   fprintf(ptr_out,"%ld %ld M 180 rotate\n",my,mx);
   fprintf(ptr_out,"%s show\n",cp);
   fprintf(ptr_out,"grestore\n");
  }


if (g_y2label[0] != '\0')        /* Y2 Label */
  {
   lstr = convert_string_to_ps(g_y2label,cp);

   mx  = graph_x_high + 4600;
   my  = (graph_y_low + graph_y_high) / 2;
   my -= lstr / 2;

   fprintf(ptr_out,"gsave\n");
   fprintf(ptr_out,"%ld %ld M 0 rotate\n",my,mx); 
   fprintf(ptr_out,"%s show\n",cp);
   fprintf(ptr_out,"grestore\n");
  }


if (g_title[0] != '\0')        /* Title */
  {
   lstr = convert_string_to_ps(g_title,cp);

   mx  = (graph_x_low + graph_x_high) / 2;
   mx -= lstr / 2;
   my  = graph_y_high - 3200;

   fprintf(ptr_out,"gsave\n");
   fprintf(ptr_out,"%ld %ld M 90 rotate\n",my,mx);
   fprintf(ptr_out,"%s show\n",cp);
   fprintf(ptr_out,"grestore\n");
  }

/*
 *      use smaller font for the labels
 */

fprintf(ptr_out,"/Helvetica findfont 1000 scalefont setfont\n");

if (g_label1[0] != '\0')        /* Label 1 */
  {
   convert_string_to_ps(g_label1,cp);

   mx = (long)(g_label1_x * 7200.0 + 0.5);
   my = (long)(7200.0 * (g_papersize_width - g_label1_y) + 0.5);

   fprintf(ptr_out,"gsave\n");
   fprintf(ptr_out,"%ld %ld M 90 rotate\n",my,mx);
   fprintf(ptr_out,"%s show\n",cp);
   fprintf(ptr_out,"grestore\n");
  }


if (g_label2[0] != '\0')        /* Label 2 */
  {
   convert_string_to_ps(g_label2,cp);

   mx = (long)(g_label2_x * 7200.0 + 0.5);
   my = (long)(7200.0 * (g_papersize_width - g_label2_y) + 0.5);

   fprintf(ptr_out,"gsave\n");
   fprintf(ptr_out,"%ld %ld M 90 rotate\n",my,mx);
   fprintf(ptr_out,"%s show\n",cp);
   fprintf(ptr_out,"grestore\n");
  }


if (g_label3[0] != '\0')        /* Label 3 */
  {
   convert_string_to_ps(g_label3,cp);

   mx = (long)(g_label3_x * 7200.0 + 0.5);
   my = (long)(7200.0 * (g_papersize_width - g_label3_y) + 0.5);

   fprintf(ptr_out,"gsave\n");
   fprintf(ptr_out,"%ld %ld M 90 rotate\n",my,mx);
   fprintf(ptr_out,"%s show\n",cp);
   fprintf(ptr_out,"grestore\n");
  }


if (g_label4[0] != '\0')        /* Label 4 */
  {
   convert_string_to_ps(g_label4,cp);

   mx = (long)(g_label4_x * 7200.0 + 0.5);
   my = (long)(7200.0 * (g_papersize_width - g_label4_y) + 0.5);

   fprintf(ptr_out,"gsave\n");
   fprintf(ptr_out,"%ld %ld M 90 rotate\n",my,mx);
   fprintf(ptr_out,"%s show\n",cp);
   fprintf(ptr_out,"grestore\n");
  }


if (g_label5[0] != '\0')        /* Label 5 */
  {
   convert_string_to_ps(g_label5,cp);

   mx = (long)(g_label5_x * 7200.0 + 0.5);
   my = (long)(7200.0 * (g_papersize_width - g_label5_y) + 0.5);

   fprintf(ptr_out,"gsave\n");
   fprintf(ptr_out,"%ld %ld M 90 rotate\n",my,mx);
   fprintf(ptr_out,"%s show\n",cp);
   fprintf(ptr_out,"grestore\n");
  }


if (g_label6[0] != '\0')        /* Label 6 */
  {
   convert_string_to_ps(g_label6,cp);

   mx = (long)(g_label6_x * 7200.0 + 0.5);
   my = (long)(7200.0 * (g_papersize_width - g_label6_y) + 0.5);

   fprintf(ptr_out,"gsave\n");
   fprintf(ptr_out,"%ld %ld M 90 rotate\n",my,mx);
   fprintf(ptr_out,"%s show\n",cp);
   fprintf(ptr_out,"grestore\n");
  }


if (g_label7[0] != '\0')        /* Label 7 */
  {
   convert_string_to_ps(g_label7,cp);

   mx = (long)(g_label7_x * 7200.0 + 0.5);
   my = (long)(7200.0 * (g_papersize_width - g_label7_y) + 0.5);

   fprintf(ptr_out,"gsave\n");
   fprintf(ptr_out,"%ld %ld M 90 rotate\n",my,mx);
   fprintf(ptr_out,"%s show\n",cp);
   fprintf(ptr_out,"grestore\n");
  }


if (g_label8[0] != '\0')        /* Label 8 */
  {
   convert_string_to_ps(g_label8,cp);

   mx = (long)(g_label8_x * 7200.0 + 0.5);
   my = (long)(7200.0 * (g_papersize_width - g_label8_y) + 0.5);

   fprintf(ptr_out,"gsave\n");
   fprintf(ptr_out,"%ld %ld M 90 rotate\n",my,mx);
   fprintf(ptr_out,"%s show\n",cp);
   fprintf(ptr_out,"grestore\n");
  }


/*
 *      draw the GRID lines
 */

log_flag = FALSE;
if (g_xaxis_scale == X_AXIS_SCALE_LOG)
  log_flag = TRUE;

if (g_gridlines)
  {
   fprintf(ptr_out,"50 setlinewidth\n");

   dash_on  = 900;       /* points the dash is ON  */
   dash_off = 450;       /* points the dash is OFF */

   if (g_xaxis_scale == X_AXIS_SCALE_LINEAR)
     {
      jdiv  = g_x_divisions;
      delta = ((double)graph_x_high - graph_x_low) / jdiv;

      for (j=1; j < jdiv; j++)                 /* loop for all divisions */
        {
         xcoord = (long)(delta * j + graph_x_low + 0.5);
         ycoord = graph_y_high;

         while (1)        /* draw incremental line segments */
           {
            fprintf(ptr_out,"N %ld %ld M ",ycoord,xcoord);

            ycoord += dash_on;
            if (ycoord >= graph_y_low)
               ycoord = graph_y_low;

            fprintf(ptr_out,"%ld %ld L S\n",ycoord,xcoord);

            ycoord += dash_off;

            if (ycoord >= graph_y_low)
              break;
           }
        }
     }

   if (g_xaxis_scale == X_AXIS_SCALE_LOG)
     {
      if (g_true_x_min <= 0.0 || g_true_x_max <= 0.0)
        {
         fprintf(stderr,
	   "ERROR : There is X axis data less than 0 on a LOG plot\n");
         exit_program(27);
        }

      g_graph_x_min = floor(log10(g_true_x_min));
      g_graph_x_max = floor(log10(g_true_x_max)) + 1.0;

      /* recompute since we changed the x axis scaling */

      xslope  = (double)(graph_x_high - graph_x_low) /
                        (g_graph_x_max - g_graph_x_min);
      
      xint    = graph_x_high - xslope * g_graph_x_max;
      
      d2 = g_graph_x_max - 0.01;
      d1 = g_graph_x_min;

      while (1)          /* loop for all decades of the LOG plot */
        {
         if (d1 > d2)  break;

         c1     = pow(10.0,d1);
         cstart = c1;

         for (j=2; j <= 9; j++)  
           {
            c1 += cstart;

            xcoord = (long)(log10(c1) * xslope + xint + 0.5);
            ycoord = graph_y_high;

            while (1)        /* draw incremental line segments */
              {
               fprintf(ptr_out,"N %ld %ld M ",ycoord,xcoord);

               ycoord += dash_on;
               if (ycoord >= graph_y_low)
                  ycoord = graph_y_low;

               fprintf(ptr_out,"%ld %ld L S\n",ycoord,xcoord);

               ycoord += dash_off;

               if (ycoord >= graph_y_low)
                 break;
              }
           }

         c1 = 10.0 * cstart;        /* draw the even decade line solid */

         xcoord = (long)(log10(c1) * xslope + xint + 0.5);
         ycoord = graph_y_high;
         fprintf(ptr_out,"N %ld %ld M ",ycoord,xcoord);

         ycoord = graph_y_low;
         fprintf(ptr_out,"%ld %ld L S\n",ycoord,xcoord);

         d1 += 1.0;
        }
     }


   if (g_field_def_y1 != 0)       /* plot grid-lines for the Y range */
      jdiv = g_y1_divisions;
   else
      jdiv = g_y2_divisions;

   delta = ((double)graph_y_high - graph_y_low) / jdiv;

   for (j=1; j < jdiv; j++)                 /* loop for all divisions */
     {
      xcoord = graph_x_low;
      ycoord = (long)(delta * j + graph_y_low + 0.5);

      while (1)        /* draw incremental line segments */
        {
         fprintf(ptr_out,"N %ld %ld M ",ycoord,xcoord);

         xcoord += dash_on;
         if (xcoord >= graph_x_high)
            xcoord = graph_x_high;

         fprintf(ptr_out,"%ld %ld L S\n",ycoord,xcoord);

         xcoord += dash_off;

         if (xcoord >= graph_x_high)
           break;
        }
     }
  }


/*
 *      graph the data now
 */

bitmap = 0;

if (g_field_def_y1 != 0)
  {
   if (g_graph_type == GRAPH_TYPE_LINES)
      bitmap |= 1;

   if (g_graph_type == GRAPH_TYPE_POINTS)
      bitmap |= 2;

   if (g_graph_type == GRAPH_TYPE_LINES_AND_POINTS) 
      bitmap |= 3;
  }

if (g_field_def_y2 != 0)
  {
   if (g_graph_type == GRAPH_TYPE_LINES)
      bitmap |= 4;

   if (g_graph_type == GRAPH_TYPE_POINTS)
      bitmap |= 8;

   if (g_graph_type == GRAPH_TYPE_LINES_AND_POINTS) 
      bitmap |= 0xC;
  }

if ((ptr = fopen(g_plot_file,"r")) == NULL)
  {
   fprintf(stderr,"Could not open plot file \'%s\'\n",g_plot_file);
   exit_program(23);
  }

mask = 1;

fprintf(ptr_out,"100 setlinewidth\n");

for (i=1; i <= 4; i++)         /* may have to loop 4 times */
  {
   if (i == 3)
     {
      fprintf(ptr_out,"75 setlinewidth\n");
      fprintf(ptr_out,"0.4 setgray\n");
     }

   rewind(ptr);

   first      = TRUE;
   draw_lines = FALSE;         /* assume we will draw points */
   local_skip = 0;
   ct5        = 0;
   ct150      = 148;

   if ((bitmap & mask) == mask)
     {
      if ((i & 1) != 0)
        draw_lines = TRUE;              /* we will draw lines */

      sample_count = 0;

      if (g_usesamples)            /* using a limited set of the data ?? */
        {                          /* scan by initial samples            */
         for (m=1; m < g_usesamples_low; m++)
           {
            sample_count++;

            if (fgets(g_input_string,MAX_LINE_LENGTH,ptr) == NULL)
              {
               fclose(ptr);
               fprintf(stderr,
    "Fewer samples in data file than specified with \'set usesamples\'\n");
               exit_program(15);
              }
           }
        }

      while (fgets(g_input_string,MAX_LINE_LENGTH,ptr))
        {
         sample_count++;

         filter_input_string(g_input_string);
         field_count = parse_line_into_fields(g_input_string);
         field_count = reset_field_count(field_count);

	 if (g_field_def_x > field_count)
	   {
            fclose(ptr);
            fprintf(stderr,
	            "ERROR: Too few data fields in plot file for X axis\n");
            fprintf(stderr,
	            "Line : %s\n",g_input_string);
            exit_program(15);
	   }

	 if (g_field_def_y1 > field_count)
	   {
            fclose(ptr);
            fprintf(stderr,
	            "ERROR: Too few data fields in plot file for Y axis\n");
            fprintf(stderr,
	            "Line : %s\n",g_input_string);
            exit_program(15);
	   }

	 if (g_field_def_y2 > field_count)
	   {
            fclose(ptr);
            fprintf(stderr,
	            "ERROR: Too few data fields in plot file for Y2 axis\n");
            fprintf(stderr,
	            "Line : %s\n",g_input_string);
            exit_program(15);
	   }

         if (g_field_def_x != 0)
           xdata = atof(g_buf[g_field_def_x]) * g_xmultiplier + g_xoffset;
         else
           xdata = sample_count * g_xmultiplier + g_xoffset;

         if (log_flag)
           xdata = log10(xdata);

	 xcoord = (long)(xdata * xslope + xint + 0.5);

         if (xcoord < graph_x_low)   xcoord = graph_x_low;
         if (xcoord > graph_x_high)  xcoord = graph_x_high;

	 if (mask < 4)
	   {
            ydata = atof(g_buf[g_field_def_y1]) * g_y1multiplier + g_y1offset;
   	    ycoord = (long)(ydata * y1slope + y1int + 0.5);
	   }
         else
	   {
            ydata = atof(g_buf[g_field_def_y2]) * g_y2multiplier + g_y2offset;
   	    ycoord = (long)(ydata * y2slope + y2int + 0.5);
	   }

         if (ycoord > graph_y_low)    ycoord = graph_y_low;
         if (ycoord < graph_y_high)   ycoord = graph_y_high;

	 if (local_skip == 0)
	   {
            if (draw_lines)
	      {
	       if (first)
	         {
	          first = FALSE;
                  fprintf(ptr_out,"N %ld %ld M ",ycoord,xcoord);
	         }
	       else
	         {
                  fprintf(ptr_out,"%ld %ld L ",ycoord,xcoord);
	         }

	       if (--ct150 == 0)
	         {
		  ct150 = 150;
                  fprintf(ptr_out,"S\n");
                  fprintf(ptr_out,"N %ld %ld M ",ycoord,xcoord);
		 }
	      }
	    else		/* drawing points */
	      {
               if (i <= 2)
                 fprintf(ptr_out,"%ld %ld D ",ycoord,xcoord);
               else
                 fprintf(ptr_out,"%ld %ld B ",ycoord,xcoord);
	      }

	    if (++ct5 == 5)
	      {
	       ct5 = 0;
               fprintf(ptr_out,"\n");
	      }
	   }

         if (local_skip == 0)
	   {
            if (g_skip_factor > 0)
	      {
	       local_skip = g_skip_factor;
	      }
	   }
         else
	    local_skip--;

         if (g_usesamples == TRUE && sample_count >= g_usesamples_high)
           break;
        }

      if (draw_lines)
         fprintf(ptr_out,"S\n");
      else
        {
	 if (ct5 != 0)
           fprintf(ptr_out,"\n");
        }
     }
   mask = mask << 1;
  }

fclose(ptr);

fprintf(ptr_out,"100 setlinewidth\n");
fprintf(ptr_out,"0 setgray\n");

/*
 *      use this font for the axis numbers
 */

fprintf(ptr_out,"/Helvetica findfont 1200 scalefont setfont\n");

/*
 *	Put the numbers on the X axis 
 */

if (g_xaxis_scale == X_AXIS_SCALE_LINEAR)
  {
   jdiv  = g_x_divisions;
   delta = ((double)graph_x_high - graph_x_low) / jdiv;
   c1    = g_graph_x_min;
   c2    = (g_graph_x_max - g_graph_x_min) / jdiv;

   for (j=0; j <= jdiv; j++)   
     {
      mx = (long)(delta * j + graph_x_low + 0.5);
      my = graph_y_low;

      format_double(c1,cp);
      lstr = convert_string_to_ps(cp,cp2);

      mx -= lstr * 12 / 28;       /* lstr / 2 * (12 / 14) */
      my += 1400;
   
      fprintf(ptr_out,"gsave\n");
      fprintf(ptr_out,"%ld %ld M 90 rotate\n",my,mx);
      fprintf(ptr_out,"%s show\n",cp2);
      fprintf(ptr_out,"grestore\n");

      c1 += c2;
     }
  }

if (g_xaxis_scale == X_AXIS_SCALE_LOG)
  {
   d2 = g_graph_x_max + 0.01;
   d1 = g_graph_x_min;

   /*
    *   Only display the even powers of 10 for a LOG plot
    */

   while (1)          /* loop for all decades of the LOG plot */
     {
      if (d1 > d2)  break;

      c1 = pow(10.0,d1);

      mx = (long)(d1 * xslope + xint + 0.5);
      my = graph_y_low;

      format_double(c1,cp);
      lstr = convert_string_to_ps(cp,cp2);

      mx -= lstr * 12 / 28;       /* lstr / 2 * (12 / 14) */
      my += 1400;
   
      fprintf(ptr_out,"gsave\n");
      fprintf(ptr_out,"%ld %ld M 90 rotate\n",my,mx);
      fprintf(ptr_out,"%s show\n",cp2);
      fprintf(ptr_out,"grestore\n");

      d1 += 1.0;
     }
  }

if (g_field_def_y1 != 0)      /* put the numbers on the Y1 Axis */
  {
   jdiv  = g_y1_divisions;
   delta = ((double)graph_y_high - graph_y_low) / jdiv;
   c1    = g_graph_y1_min;
   c2    = (g_graph_y1_max - g_graph_y1_min) / jdiv;

   for (j=0; j <= jdiv; j++)                 /* loop for all divisions */
     {
      mx = graph_x_low;
      my = (long)(delta * j + graph_y_low + 0.5);

      format_double(c1,cp);
      lstr = convert_string_to_ps(cp,cp2);

      mx -= lstr * 12 / 14 + 500;
      my += 375 * 12 / 14;
   
      fprintf(ptr_out,"gsave\n");
      fprintf(ptr_out,"%ld %ld M 90 rotate\n",my,mx);
      fprintf(ptr_out,"%s show\n",cp2);
      fprintf(ptr_out,"grestore\n");

      c1 += c2;
     }
  }

if (g_field_def_y2 != 0)      /* put the numbers on the Y2 Axis */
  {
   /*
    * 	In general, we can only plot the top & bottom number of the Y2 
    *	axis. If there is no Y1 range OR the #divisions is the same as
    *	Y1, then we can label every division of Y2.
    */

   jdiv = 1;
   if (g_y1_divisions == g_y2_divisions || g_field_def_y1 == 0) 
     jdiv = g_y2_divisions;

   delta = ((double)graph_y_high - graph_y_low) / jdiv;
   c1    = g_graph_y2_min;
   c2    = (g_graph_y2_max - g_graph_y2_min) / jdiv;

   for (j=0; j <= jdiv; j++)
     {
      mx = graph_x_high;
      my = (long)(delta * j + graph_y_low + 0.5);

      format_double(c1,cp);
      lstr = convert_string_to_ps(cp,cp2);

      mx += 500;
      my += 375 * 12 / 14;
   
      fprintf(ptr_out,"gsave\n");
      fprintf(ptr_out,"%ld %ld M 90 rotate\n",my,mx);
      fprintf(ptr_out,"%s show\n",cp2);
      fprintf(ptr_out,"grestore\n");

      c1 += c2;
     }
  }

memory_free(cp);
}
/***********************************************************************/
ps_print_trailer()
{
fprintf(ptr_out,"showpage\n");
fprintf(ptr_out,"%%%%Trailer\n");
fprintf(ptr_out,"%%%%Pages: 1\n");
}
/***********************************************************************/
