/* 
 * FPLOT - fpset.c
 *
 * Copyright (C) 1997   Michael C. Ring
 *
 * Permission to use, copy, and distribute this software and its
 * documentation for any purpose with or without fee is hereby granted, 
 * provided that the above copyright notice appear in all copies and 
 * that both that copyright notice and this permission notice appear 
 * in supporting documentation.
 *
 * Permission to modify the software is granted, but not the right to
 * distribute the modified code.  Modifications are to be distributed 
 * as patches to released version.
 *  
 * This software is provided "as is" without express or implied warranty.
 */

/*
 *      fpset.c
 *
 *      $Log: fpset.c $
 *      Revision 1.3  1997/11/26 23:20:56  mring
 *      added MAX_LINE_LENGTH
 *
 *      Revision 1.2  1997/11/24 01:32:07  mring
 *      added copyright notice
 *
 *      Revision 1.1  1997/11/23 23:08:19  mring
 *      Initial revision
 *
 */

#include "fplot.h"
#include "glob.h"

/* RCS TRACER */
static  char    
RCS_ID[] = "$Id: fpset.c 1.3 1997/11/26 23:20:56 mring Exp $";


/***********************************************************************/
plot_raw_file()
{
generate_plot(RAW_FILE,0);
}
/***********************************************************************/
plot_command_file()
{
int     cmd_flag, count;

if ((g_tmp_ptr = fopen(g_tmp_file,"r")) == NULL)
  {
   fprintf(stderr,
           "ERROR : Could not open temporary file \'%s\'\n",g_tmp_file);
   exit_program(22);
  }
                                              /* loop thru all commands */
while (fgets(g_input_string,MAX_LINE_LENGTH,g_tmp_ptr))
  {
   cmd_flag = FALSE;
   filter_input_string(g_input_string);

   count = parse_line_into_fields(g_input_string);

   if (count >= 1)               /* only process commands if there is one */
     {
      lowercase(g_buf[1]);       /* convert field 1 to lowercase */

      if (g_buf[1][0] != '#')    /* comments begin with a '#' */
        {
         if (strcmp(g_buf[1],"set") == 0)   /* we have a 'set' command */
           {
            cmd_flag = TRUE;
            parse_set_command(count);
           }
   
         if (strcmp(g_buf[1],"reset") == 0) /* we have a 'reset' command */
           {
            cmd_flag = TRUE;
            parse_reset_command(count);
           }
   
         if (strcmp(g_buf[1],"plot") == 0)  /* we have a 'plot' command */
           {
            cmd_flag = TRUE;
            generate_plot(COMMAND_FILE,count);
           }

         if (cmd_flag == FALSE)
           {
            fprintf(stderr,"ERROR : Unknown command \'%s\'\n",g_buf[1]);
            exit_program(24);
           }
        }
     }
  }

fclose(g_tmp_ptr);
g_tmp_ptr = NULL;
}
/***********************************************************************/
parse_set_command(fieldcount)
int     fieldcount;
{
int     cmd_found, label_flag, fielddef;
double  d1, d2, *dx, *dy;
char    **cpp;

fieldcount = reset_field_count(fieldcount);

if (fieldcount <= 2)           /* all set commands have at least 3 fields */
  {
   invalid_set_command(fieldcount,30);
  }

lowercase(g_buf[2]);           /* find out which command */

cmd_found = FALSE;
 
/*
 *      set papersize 8.5, 11.0
 */

if (strposition(g_buf[2],"papersi") == 0) /* accept this for 'papersize' */
  {
   cmd_found = TRUE;

   if (fieldcount != 4)                   /* this command has 4 fields */
     invalid_set_command(fieldcount,31);

   g_papersize_width  = atof(g_buf[3]);
   g_papersize_length = atof(g_buf[4]);

   if (g_papersize_width < 0.9 || g_papersize_length < 0.90)
     invalid_set_command(fieldcount,31);
  }


/*
 *      set outputformat postscript
 */
                                        /* accept this for 'outputformat' */
if (strposition(g_buf[2],"outputfor") == 0)
  {
   cmd_found = TRUE;

   if (fieldcount != 3)                   /* this command has 3 fields */
     invalid_set_command(fieldcount,31);

   lowercase(g_buf[3]);

   if (strposition(g_buf[3],"postsc") == 0)    /* postscript */
      g_outputformat = FORMAT_POSTSCRIPT;
   else
      invalid_set_command(fieldcount,31);
  }


/*
 *      set usesamples 50, 100
 */

if (strposition(g_buf[2],"usesampl") == 0) /* accept this for 'usesamples' */
  {
   cmd_found = TRUE;

   if (fieldcount != 4)                   /* this command has 4 fields */
     invalid_set_command(fieldcount,31);

   g_usesamples_low  = atol(g_buf[3]);
   g_usesamples_high = atol(g_buf[4]);

   if (g_usesamples_low > g_usesamples_high)
     invalid_set_command(fieldcount,31);

   if (g_usesamples_low < 0 || g_usesamples_high < 0)
     invalid_set_command(fieldcount,31);

   g_usesamples = TRUE;
  }


/*
 *      set skipfactor 9
 */

if (strposition(g_buf[2],"skipfa") == 0) /* accept this for 'skipfactor' */
  {
   cmd_found = TRUE;

   if (fieldcount != 3)                   /* this command has 3 fields */
     invalid_set_command(fieldcount,31);

   if ((g_skip_factor = atol(g_buf[3])) < 0)
     invalid_set_command(fieldcount,31);
  }


/*
 *      set gridlines on
 *      set gridlines off
 */

if (strposition(g_buf[2],"gridli") == 0) /* accept this for 'gridlines' */
  {
   cmd_found = TRUE;

   if (fieldcount != 3)                   /* this command has 3 fields */
     invalid_set_command(fieldcount,31);

   lowercase(g_buf[3]);

   if (strcmp(g_buf[3],"on") == 0)
      g_gridlines = TRUE;
   else
     {
      if (strcmp(g_buf[3],"off") == 0)
        g_gridlines = FALSE;
      else
        invalid_set_command(fieldcount,31);
     }
  }


/*
 *      set xoffset -2.0
 */

if (strposition(g_buf[2],"xoffs") == 0) /* accept this for 'xoffset' */
  {
   cmd_found = TRUE;

   if (fieldcount != 3)                   /* this command has 3 fields */
     invalid_set_command(fieldcount,31);

   g_xoffset = atof(g_buf[3]);
  }


/*
 *      set yoffset 32.0    (also y1offset)
 */

if (strposition(g_buf[2],"yoffs") == 0 ||
    strposition(g_buf[2],"y1offs") == 0)
  {
   cmd_found = TRUE;

   if (fieldcount != 3)                   /* this command has 3 fields */
     invalid_set_command(fieldcount,31);

   g_y1offset = atof(g_buf[3]);
  }


/*
 *      set y2offset -32.0  
 */

if (strposition(g_buf[2],"y2offs") == 0)
  {
   cmd_found = TRUE;

   if (fieldcount != 3)                   /* this command has 3 fields */
     invalid_set_command(fieldcount,31);

   g_y2offset = atof(g_buf[3]);
  }


/*
 *      set xmultiplier 1000.0
 */

if (strposition(g_buf[2],"xmulti") == 0) /* accept this for 'xmultiplier' */
  {
   cmd_found = TRUE;

   if (fieldcount != 3)                   /* this command has 3 fields */
     invalid_set_command(fieldcount,31);

   g_xmultiplier = atof(g_buf[3]);
  }


/*
 *      set ymultiplier 1.0E6      (also y1multiplier)
 */

if (strposition(g_buf[2],"ymulti") == 0 ||
    strposition(g_buf[2],"y1multi") == 0)
  {
   cmd_found = TRUE;

   if (fieldcount != 3)                   /* this command has 3 fields */
     invalid_set_command(fieldcount,31);

   g_y1multiplier = atof(g_buf[3]);
  }


/*
 *      set y2multiplier 10.0     
 */

if (strposition(g_buf[2],"y2multi") == 0)
  {
   cmd_found = TRUE;

   if (fieldcount != 3)                   /* this command has 3 fields */
     invalid_set_command(fieldcount,31);

   g_y2multiplier = atof(g_buf[3]);
  }


/*
 *      set fielddefinition x,1
 *      set fielddefinition y,2
 *      set fielddefinition y1,2
 *      set fielddefinition y2,3
 */
                                  /* accept this for 'fielddefinition' */
if (strposition(g_buf[2],"fielddef") == 0)
  {
   cmd_found = TRUE;

   if (fieldcount != 4)                   /* this command has 4 fields */
     invalid_set_command(fieldcount,31);

   lowercase(g_buf[3]);
   fielddef = FALSE;

   if (strcmp(g_buf[3],"x") == 0)
     {
      fielddef = TRUE;
      if ((g_field_def_x = atoi(g_buf[4])) < 1)
        invalid_set_command(fieldcount,31);
     }

   if (strcmp(g_buf[3],"y") == 0 || strcmp(g_buf[3],"y1") == 0)
     {
      fielddef = TRUE;
      if ((g_field_def_y1 = atoi(g_buf[4])) < 1)
        invalid_set_command(fieldcount,31);
     }

   if (strcmp(g_buf[3],"y2") == 0)
     {
      fielddef = TRUE;
      if ((g_field_def_y2 = atoi(g_buf[4])) < 1)
        invalid_set_command(fieldcount,31);
     }

   if (fielddef == FALSE)
     {
      invalid_set_command(fieldcount,31);
     }
  }


/*
 *      set xaxisscale linear
 *      set xaxisscale log
 */

if (strposition(g_buf[2],"xaxissc") == 0) /* accept this for 'xaxisscale' */
  {
   cmd_found = TRUE;

   if (fieldcount != 3)                   /* this command has 3 fields */
     invalid_set_command(fieldcount,31);

   lowercase(g_buf[3]);

   if (strcmp(g_buf[3],"linear") == 0)
      g_xaxis_scale = X_AXIS_SCALE_LINEAR;
   else
     {
      if (strcmp(g_buf[3],"log") == 0)
        g_xaxis_scale = X_AXIS_SCALE_LOG;
      else
        invalid_set_command(fieldcount,31);
     }
  }


/*
 *      set graphtype lines
 *      set graphtype points
 *      set graphtype linesandpoints
 */

if (strposition(g_buf[2],"grapht") == 0) /* accept this for 'graphtype' */
  {
   cmd_found = TRUE;

   if (fieldcount != 3)                   /* this command has 3 fields */
     invalid_set_command(fieldcount,31);

   lowercase(g_buf[3]);

   if (strcmp(g_buf[3],"lines") == 0)
      g_graph_type = GRAPH_TYPE_LINES;
   else
     {
      if (strcmp(g_buf[3],"points") == 0)
         g_graph_type = GRAPH_TYPE_POINTS;
      else
        {
         if (strcmp(g_buf[3],"linesandpoints") == 0)
           g_graph_type = GRAPH_TYPE_LINES_AND_POINTS;
         else
           invalid_set_command(fieldcount,31);
        }
     }
  }


/*
 *      set xrange autorange
 *      set xrange -35.0,45.0
 */

if (strcmp(g_buf[2],"xrange") == 0)
  {
   cmd_found = TRUE;

   if (fieldcount == 3)
     {
      lowercase(g_buf[3]);

      if (strposition(g_buf[3],"auto") == 0)
         g_xrange_auto = TRUE;
      else
         invalid_set_command(fieldcount,31);
     }
   else
     {
      if (fieldcount == 4)
        {
         g_xrange_auto = FALSE;

         g_xrange_low  = atof(g_buf[3]);
         g_xrange_high = atof(g_buf[4]);

         adjust_min_max(&g_xrange_low,&g_xrange_high);

         d1 = fabs(g_xrange_low);
         d2 = fabs(g_xrange_high);

         if (d2 < d1)  d2 = d1;

         if (d2 < 1.0E-12)
           invalid_set_command(fieldcount,31);

         d1 = (g_xrange_high - g_xrange_low) / d2;

         if (fabs(d1) < 1.0E-12)
           invalid_set_command(fieldcount,31);
        }
      else
        {
         invalid_set_command(fieldcount,31);
        }
     }
  }


/*
 *      set yrange autorange       (y1range also)
 *      set yrange -35.0,45.0      
 */

if (strcmp(g_buf[2],"yrange") == 0 || strcmp(g_buf[2],"y1range") == 0)
  {
   cmd_found = TRUE;

   if (fieldcount == 3)
     {
      lowercase(g_buf[3]);

      if (strposition(g_buf[3],"auto") == 0)
         g_y1range_auto = TRUE;
      else
         invalid_set_command(fieldcount,31);
     }
   else
     {
      if (fieldcount == 4)
        {
         g_y1range_auto = FALSE;

         g_y1range_low  = atof(g_buf[3]);
         g_y1range_high = atof(g_buf[4]);

         adjust_min_max(&g_y1range_low,&g_y1range_high);

         d1 = fabs(g_y1range_low);
         d2 = fabs(g_y1range_high);

         if (d2 < d1)  d2 = d1;

         if (d2 < 1.0E-12)
           invalid_set_command(fieldcount,31);

         d1 = (g_y1range_high - g_y1range_low) / d2;

         if (fabs(d1) < 1.0E-12)
           invalid_set_command(fieldcount,31);
        }
      else
        {
         invalid_set_command(fieldcount,31);
        }
     }
  }


/*
 *      set y2range autorange
 *      set y2range -35.0,45.0      
 */

if (strcmp(g_buf[2],"y2range") == 0)
  {
   cmd_found = TRUE;

   if (fieldcount == 3)
     {
      lowercase(g_buf[3]);

      if (strposition(g_buf[3],"auto") == 0)
         g_y2range_auto = TRUE;
      else
         invalid_set_command(fieldcount,31);
     }
   else
     {
      if (fieldcount == 4)
        {
         g_y2range_auto = FALSE;

         g_y2range_low  = atof(g_buf[3]);
         g_y2range_high = atof(g_buf[4]);

         adjust_min_max(&g_y2range_low,&g_y2range_high);

         d1 = fabs(g_y2range_low);
         d2 = fabs(g_y2range_high);

         if (d2 < d1)  d2 = d1;

         if (d2 < 1.0E-12)
           invalid_set_command(fieldcount,31);

         d1 = (g_y2range_high - g_y2range_low) / d2;

         if (fabs(d1) < 1.0E-12)
           invalid_set_command(fieldcount,31);
        }
      else
        {
         invalid_set_command(fieldcount,31);
        }
     }
  }


/*
 *      set outputfile "C:\data.tmp"
 */

if (strcmp(g_buf[2],"outputfile") == 0)
  {
   cmd_found = TRUE;

   if (fieldcount != 3)                   /* this command has 3 fields */
     invalid_set_command(fieldcount,31);

   memory_free(g_output_file);
   g_output_file = memory_allocate((strlen(g_buf[3]) + 4) * sizeof(char));
   strcpy(g_output_file,g_buf[3]);
  }


/*
 *      set title "this is the graph title"
 */

if (strcmp(g_buf[2],"title") == 0)
  {
   cmd_found = TRUE;

   if (fieldcount != 3)                   /* this command has 3 fields */
     invalid_set_command(fieldcount,31);

   memory_free(g_title);
   g_title = memory_allocate((strlen(g_buf[3]) + 4) * sizeof(char));
   strcpy(g_title,g_buf[3]);
  }


/*
 *      set xlabel "x axis label"
 */

if (strcmp(g_buf[2],"xlabel") == 0)
  {
   cmd_found = TRUE;

   if (fieldcount != 3)                   /* this command has 3 fields */
     invalid_set_command(fieldcount,31);

   memory_free(g_xlabel);
   g_xlabel = memory_allocate((strlen(g_buf[3]) + 4) * sizeof(char));
   strcpy(g_xlabel,g_buf[3]);
  }


/*
 *      set ylabel "y axis label"        (also y1label)
 */

if (strcmp(g_buf[2],"ylabel") == 0 || strcmp(g_buf[2],"y1label") == 0)
  {
   cmd_found = TRUE;

   if (fieldcount != 3)                   /* this command has 3 fields */
     invalid_set_command(fieldcount,31);

   memory_free(g_y1label);
   g_y1label = memory_allocate((strlen(g_buf[3]) + 4) * sizeof(char));
   strcpy(g_y1label,g_buf[3]);
  }


/*
 *      set y2label "y2 axis label"      
 */

if (strcmp(g_buf[2],"y2label") == 0)
  {
   cmd_found = TRUE;

   if (fieldcount != 3)                   /* this command has 3 fields */
     invalid_set_command(fieldcount,31);

   memory_free(g_y2label);
   g_y2label = memory_allocate((strlen(g_buf[3]) + 4) * sizeof(char));
   strcpy(g_y2label,g_buf[3]);
  }


label_flag = FALSE;

/*
 *      set label1 3.0, 4.5, "extra label 1 at 3.0, 4.5"
 */

if (strcmp(g_buf[2],"label1") == 0)
  {
   cmd_found  = TRUE;
   label_flag = TRUE;
   cpp        = &g_label1;
   dx         = &g_label1_x;
   dy         = &g_label1_y;
  }


/*
 *      set label2 3.0, 4.5, "extra label 2 at 3.0, 4.5"
 */

if (strcmp(g_buf[2],"label2") == 0)
  {
   cmd_found  = TRUE;
   label_flag = TRUE;
   cpp        = &g_label2;
   dx         = &g_label2_x;
   dy         = &g_label2_y;
  }


/*
 *      set label3 3.0, 4.5, "extra label 3 at 3.0, 4.5"
 */

if (strcmp(g_buf[2],"label3") == 0)
  {
   cmd_found  = TRUE;
   label_flag = TRUE;
   cpp        = &g_label3;
   dx         = &g_label3_x;
   dy         = &g_label3_y;
  }


/*
 *      set label4 3.0, 4.5, "extra label 4 at 3.0, 4.5"
 */

if (strcmp(g_buf[2],"label4") == 0)
  {
   cmd_found  = TRUE;
   label_flag = TRUE;
   cpp        = &g_label4;
   dx         = &g_label4_x;
   dy         = &g_label4_y;
  }


/*
 *      set label5 3.0, 4.5, "extra label 5 at 3.0, 4.5"
 */

if (strcmp(g_buf[2],"label5") == 0)
  {
   cmd_found  = TRUE;
   label_flag = TRUE;
   cpp        = &g_label5;
   dx         = &g_label5_x;
   dy         = &g_label5_y;
  }


/*
 *      set label6 3.0, 4.5, "extra label 6 at 3.0, 4.5"
 */

if (strcmp(g_buf[2],"label6") == 0)
  {
   cmd_found  = TRUE;
   label_flag = TRUE;
   cpp        = &g_label6;
   dx         = &g_label6_x;
   dy         = &g_label6_y;
  }


/*
 *      set label7 3.0, 4.5, "extra label 7 at 3.0, 4.5"
 */

if (strcmp(g_buf[2],"label7") == 0)
  {
   cmd_found  = TRUE;
   label_flag = TRUE;
   cpp        = &g_label7;
   dx         = &g_label7_x;
   dy         = &g_label7_y;
  }


/*
 *      set label8 3.0, 4.5, "extra label 8 at 3.0, 4.5"
 */

if (strcmp(g_buf[2],"label8") == 0)
  {
   cmd_found  = TRUE;
   label_flag = TRUE;
   cpp        = &g_label8;
   dx         = &g_label8_x;
   dy         = &g_label8_y;
  }

if (label_flag == TRUE)
  {
   if (fieldcount != 5)                   /* label commands have 5 fields */
     invalid_set_command(fieldcount,31);

   *dx = atof(g_buf[3]);
   *dy = atof(g_buf[4]);

   memory_free(*cpp);
   *cpp = memory_allocate((strlen(g_buf[5]) + 4) * sizeof(char));
   strcpy(*cpp,g_buf[5]);
  }

if (cmd_found == FALSE)
  {
   invalid_set_command(fieldcount,28);
  }
}
/***********************************************************************/
parse_reset_command(fieldcount)
int     fieldcount;
{
fieldcount = reset_field_count(fieldcount);

if (fieldcount != 2)
  {
   fprintf(stderr,"ERROR : Invalid \'reset\' command\n");
   exit_program(40);
  }

lowercase(g_buf[2]);

if (strposition(g_buf[2],"defa") != 0)       /* accept this as 'defaults' */
  {
   fprintf(stderr,"ERROR : Unknown \'reset\' command\n");
   exit_program(42);
  }

/* all is good with 'reset defaults', so do it */

init_globals();
}
/***********************************************************************/
generate_plot(caller,fieldcount)
int     caller, fieldcount;
{
int     p_x, p_y1, p_y2, num_fields;

if (caller == COMMAND_FILE)
  {
   fieldcount = reset_field_count(fieldcount);
   
   if (fieldcount != 2)
     {
      fprintf(stderr,"ERROR : Invalid \'plot\' command\n");
      exit_program(40);
     }
   
   strcpy(g_plot_file,g_buf[2]);
  }

if (caller == RAW_FILE)
  {
   strcpy(g_plot_file,g_tmp_file);
  }

p_x  = g_field_def_x;            /* store the field numbers of each range */
p_y1 = g_field_def_y1;
p_y2 = g_field_def_y2;

if (p_y1 == 0 && p_y2 == 0)      /* no y-ranges specified */
  {
   if (p_x == 0)                 /* no x-range either, go figure it out */
     {
      num_fields = auto_parse_field_defs(g_plot_file);

      /*
       *    1 field , plot y from field 1    (y1)
       *    2 fields, plot x from field 1 & y from field 2
       *    3 fields, plot x from field 1, y1 from field 2 & y2 from field 3
       */

      if (num_fields == 1)
        p_y1 = 1;                 /* field 1 */

      if (num_fields == 2)
        {
	 p_x  = 1;                /* field 1 */
         p_y1 = 2;                /* field 2 */
        }

      if (num_fields == 3)
        {
         p_x  = 1;                /* field 1 */
         p_y1 = 2;                /* field 2 */
         p_y2 = 3;                /* field 3 */
        }
     }
   else                          /* must have a Y if you have an X */
     {
      fprintf(stderr,"ERROR : No Y range specified in command file\n");
      exit_program(26);
     }
  }

/*
 *      now check for auto-ranging on X, Y1, & Y2
 */

auto_scale_axis(g_plot_file,p_x,p_y1,p_y2);

g_field_def_x  = p_x;   
g_field_def_y1 = p_y1;
g_field_def_y2 = p_y2;

if (g_outputformat == FORMAT_POSTSCRIPT)
  output_postscript_plot();

if (g_outputformat == FORMAT_HPGL2)
  output_hpgl2_plot();

if (g_outputformat == FORMAT_EPSON_LQ)
  output_epson_lq_plot();

if (g_outputformat == FORMAT_EPSON_FX)
  output_epson_fx_plot();
}
/***********************************************************************/
invalid_set_command(fieldcount,code)
int     fieldcount, code;
{
int     i;

fprintf(stderr,"ERROR : Invalid \'set\' command\n");

for (i=1; i <= fieldcount; i++)
  {
   fprintf(stderr,"%s ",g_buf[i]);
  }

fprintf(stderr,"\n");
exit_program(code);
}
/***********************************************************************/
/*
 *      this function will read the data file to determine what
 *      to plot
 *
 *      function returns  :  1   1 field found,  plot y range
 *                           2   2 fields found, plot x & y1 ranges
 *                           3   3 fields found, plot x, y1, y2 ranges
 *
 *      if 1, 2, or 3 fields are not found, display an error message
 *      and exit the program.
 */
auto_parse_field_defs(filename)
char    *filename;
{
int     i, count, fieldcount;
double  sum;
FILE    *ptr;

if ((ptr = fopen(filename,"r")) == NULL)
  {
   fprintf(stderr,"Could not open plot file \'%s\'\n",filename);
   exit_program(12);
  }

sum   = 0.0;
count = 0;

for (i=1; i <= 20; i++)		/* read the first 20 lines of the file */
  {
   if (fgets(g_input_string,MAX_LINE_LENGTH,ptr) == NULL)  break;

   count++;
   filter_input_string(g_input_string);

   fieldcount = parse_line_into_fields(g_input_string);
   fieldcount = reset_field_count(fieldcount);

   sum += (double)fieldcount;
  }

fclose(ptr);

sum /= count;

for (i=1; i <= 3; i++)        /* check for the 3 allowable formats */
  {
   if (fabs(sum - i) < 0.1)
     return (i);
  }

fprintf(stderr,"Unable to determine which fields to plot\n");
exit_program(12);
}
/***********************************************************************/
auto_scale_axis(filename,fieldnum_x,fieldnum_y1,fieldnum_y2)
char    *filename;
int     fieldnum_x, fieldnum_y1, fieldnum_y2;
{
int     i, field_count;
long    m, mstart, mstop, sample_count;
double  lo, hi, x_data, y1_data, y2_data, x_max, x_min, 
	y1_max, y1_min, y2_max, y2_min;
FILE    *ptr;      

x_min  =  1.0E200;
x_max  = -1.0E200;
y1_min =  1.0E200;
y1_max = -1.0E200;
y2_min =  1.0E200;
y2_max = -1.0E200;

g_x_divisions  = 16;
g_y1_divisions = 10;
g_y2_divisions = 10;

i = 0;

if (g_xrange_auto == FALSE)
  {
   i++;
   x_min = g_xrange_low;
   x_max = g_xrange_high;

   lo = x_min;
   hi = x_max;

   g_true_x_min = lo;
   g_true_x_max = hi;

   scale_values(TRUE,&lo,&hi,&g_x_divisions);
  }

if (g_y1range_auto == FALSE)
  {
   i++;
   y1_min = g_y1range_low;
   y1_max = g_y1range_high;

   lo = y1_min;
   hi = y1_max;
   scale_values(TRUE,&lo,&hi,&g_y1_divisions);
  }

if (g_y2range_auto == FALSE)
  {
   i++;
   y2_min = g_y2range_low;
   y2_max = g_y2range_high;

   lo = y2_min;
   hi = y2_max;
   scale_values(TRUE,&lo,&hi,&g_y2_divisions);
  }

if (i == 3)         /* all 3 axis's were specified, return now */
  {
   g_graph_x_min  = x_min;
   g_graph_x_max  = x_max;
   g_graph_y1_min = y1_min;
   g_graph_y1_max = y1_max;
   g_graph_y2_min = y2_min;
   g_graph_y2_max = y2_max;
   return(0);
  }

if ((ptr = fopen(filename,"r")) == NULL)
  {
   fprintf(stderr,"Could not open plot file \'%s\'\n",filename);
   exit_program(14);
  }

sample_count = 0;

if (g_usesamples)            /* using a limited set of the data ?? */
  {                          /* scan by initial samples            */
   for (m=1; m < g_usesamples_low; m++)
     {
      sample_count++;

      if (fgets(g_input_string,MAX_LINE_LENGTH,ptr) == NULL)
        {
         fclose(ptr);
         fprintf(stderr,
      "Fewer samples in data file than specified with \'set usesamples\'\n");
         exit_program(15);
        }
     }
  }

mstart = sample_count + 1;

while (fgets(g_input_string,MAX_LINE_LENGTH,ptr))
  {
   sample_count++;

   filter_input_string(g_input_string);
   field_count = parse_line_into_fields(g_input_string);
   field_count = reset_field_count(field_count);

   if (g_xrange_auto == TRUE && fieldnum_x != 0)
     {
      if (fieldnum_x > field_count)
        {
         fclose(ptr);
         fprintf(stderr,
                 "Invalid field in data file, field %d specified for X,\n",
                 fieldnum_x);

         fprintf(stderr,"file only has %d fields : line # %ld\n",
                         field_count,sample_count);
         exit_program(16);
	}
      
      x_data = atof(g_buf[fieldnum_x]) * g_xmultiplier + g_xoffset;
      if (x_data < x_min)  x_min = x_data;
      if (x_data > x_max)  x_max = x_data;
     }

   if (g_y1range_auto == TRUE && fieldnum_y1 != 0)
     {
      if (fieldnum_y1 > field_count)
        {
         fclose(ptr);
         fprintf(stderr,
                 "Invalid field in data file, field %d specified for Y,\n",
                 fieldnum_y1);

         fprintf(stderr,"file only has %d fields : line # %ld\n",
                         field_count,sample_count);
         exit_program(16);
	}
      
      y1_data = atof(g_buf[fieldnum_y1]) * g_y1multiplier + g_y1offset;
      if (y1_data < y1_min)  y1_min = y1_data;
      if (y1_data > y1_max)  y1_max = y1_data;
     }

   if (g_y2range_auto == TRUE && fieldnum_y2 != 0)
     {
      if (fieldnum_y2 > field_count)
        {
         fclose(ptr);
         fprintf(stderr,
                 "Invalid field in data file, field %d specified for Y2,\n",
                 fieldnum_y2);

         fprintf(stderr,"file only has %d fields : line # %ld\n",
                         field_count,sample_count);
         exit_program(16);
	}
      
      y2_data = atof(g_buf[fieldnum_y2]) * g_y2multiplier + g_y2offset;
      if (y2_data < y2_min)  y2_min = y2_data;
      if (y2_data > y2_max)  y2_max = y2_data;
     }

   if (g_usesamples == TRUE && sample_count == g_usesamples_high)
     break;
  }

fclose(ptr);

mstop = sample_count;

if (mstart > mstop)
  {
    fprintf(stderr,
    "Fewer samples in data file than specified with \'set usesamples\'\n");
    exit_program(15);
  }

if (g_xrange_auto == TRUE && fieldnum_x == 0)
  {
   x_min = (double)mstart * g_xmultiplier + g_xoffset;
   x_max = (double)mstop  * g_xmultiplier + g_xoffset;
  }

g_true_x_min = x_min;
g_true_x_max = x_max;

if (g_xrange_auto == TRUE)
  scale_values(FALSE,&x_min,&x_max,&g_x_divisions);

if (g_y1range_auto == TRUE && fieldnum_y1 != 0)
  scale_values(FALSE,&y1_min,&y1_max,&g_y1_divisions);

if (g_y2range_auto == TRUE && fieldnum_y2 != 0)
  scale_values(FALSE,&y2_min,&y2_max,&g_y2_divisions);

/*
 *      If we are plotting 2 ranges, adjust the Y1 or Y2 divisions
 *      so they are the same. Then we can label every division for
 *      both the Y1 and Y2 ranges. We can only do this if at least
 *      one of the ranges is auto-range. If both ranges are set by
 *      a set command, we will assume that is what the user wants.
 */

if (fieldnum_y1 != 0 && fieldnum_y2 != 0)   /* both ranges being used ? */
  {
   if (g_y1_divisions != g_y2_divisions)    /* not already the same ?   */
     {
      if (g_y1range_auto == TRUE && g_y2range_auto == TRUE)
        {
         if (g_y1_divisions < g_y2_divisions)	/* move Y1 */
	   {
            rescale_divisions(g_y2_divisions,g_y1_divisions,&y1_min,&y1_max);
            g_y1_divisions = g_y2_divisions;
	   }
         else					/* move Y2 */
	   {
            rescale_divisions(g_y1_divisions,g_y2_divisions,&y2_min,&y2_max);
            g_y2_divisions = g_y1_divisions;
	   }
        }

      /*  we can only move Y2 divisions since Y1 is being set by the user */

      if (g_y1range_auto == FALSE && g_y2range_auto == TRUE)
        {
         rescale_divisions(g_y1_divisions,g_y2_divisions,&y2_min,&y2_max);
         g_y2_divisions = g_y1_divisions;
        }

      /*  we can only move Y1 divisions since Y2 is being set by the user */

      if (g_y1range_auto == TRUE && g_y2range_auto == FALSE)
        {
         rescale_divisions(g_y2_divisions,g_y1_divisions,&y1_min,&y1_max);
         g_y1_divisions = g_y2_divisions;
        }
     }
  }
                                                        
g_graph_x_min  = x_min;
g_graph_x_max  = x_max;
g_graph_y1_min = y1_min;
g_graph_y1_max = y1_max;
g_graph_y2_min = y2_min;
g_graph_y2_max = y2_max;
}
/***********************************************************************/
/* 
   parse a line delimited by spaces, tabs or commas. 
   build an array of individual strings in global 'g_buf'. 

   chars between double quotes are considered part of one field. 
   spaces, tabs, and commas will be included in the field (everything 
   up to the closing double quote).

   function returns number of 'fields' converted.  (>= 1)

   OR

   function returns 0 if line is blank, i.e. line only consists of spaces, 
   tabs, commas, or newline.

   function returns -2 if max number of strings exceeded. (80 strings)

   function returns -3 if the length of any field is too long (256 char each) 
*/ 
parse_line_into_fields(s)
char    *s; 
{ 
int     quote_flag, k, ct; 
char	*p, ch; 

p          = s; 
ct         = 0; 
quote_flag = FALSE;
 
while (1) 
  { 
   ch = *p;
   if (ch == '\0' || ch == 0x0D || ch == '\n')  return(ct); 

   if (ch == ' ' || ch == ',' || ch == '\t') 
      p++; 
   else 
     { 
      if (++ct == 80)  return(-2); 

      k = -1; 

      if (ch == '\"')  /* found double quote */
	{
         quote_flag = TRUE;
	 p++;
	}

      while (1)          /* build the string for each entry */ 
	{ 
	 if (++k == 256)  return(-3);

	 ch = *p++; 
	 g_buf[ct][k] = ch;
 
	 if (ch == '\0' || ch == 0x0D || ch == '\n') 
	   { 
	    g_buf[ct][k] = '\0'; 
	    return(ct); 
	   } 

         if (quote_flag == FALSE)
	   {
	    if (ch == ' ' || ch == ',' || ch == '\t')  break; 
	   }
         else
	   {
	    if (ch == '\"')   /* found closing double quote */
	      {
	       quote_flag = FALSE;
	       break;
	      }
	   }
	} 
 
      g_buf[ct][k] = '\0';
     } 
  } 
} 
/***********************************************************************/ 
/*
 *      if a field begins with a '#', it's the start
 *      of a comment.
 */
reset_field_count(count)
int     count;
{
int     k;

for (k=1; k <= count; k++)
  {
   if (g_buf[k][0] == '#')
     return (k-1);
  }

return (count);
}
/***********************************************************************/ 
